
#ifndef __LIST_H
#define __LIST_H 1


/*
 * Simple doubly linked list implementation.
 *
 * Some of the internal functions ("__xxx") are useful when
 * manipulating whole lists rather than single entries, as
 * sometimes we already know the next/prev entries and we can
 * generate better code by using them directly rather than
 * using the generic single-entry routines.
 */

#define LIST_HEAD_INIT(name) { &(name), &(name) }

#define LIST_HEAD(name) \
	struct list_head name = LIST_HEAD_INIT(name)

#define INIT_LIST_HEAD(ptr) do { \
	(ptr)->next = (ptr); (ptr)->prev = (ptr); \
} while (0)

 
struct list_head {
  list_head *next, *prev;
  void Init(void){ next=this; prev=this; };
};

/*
 * Insert a new entry between two known consecutive entries. 
 *
 * This is only for internal list manipulation where we know
 * the prev/next entries already!
 */
inline void __list_add(list_head * enew,
	list_head * prev,
	list_head * next)
{
	next->prev = enew;
	enew->next = next;
	enew->prev = prev;
	prev->next = enew;
}

/**
 * list_add - add a new entry
 * @new: new entry to be added
 * @head: list head to add it after
 *
 * Insert a new entry after the specified head.
 * This is good for implementing stacks.
 */
inline void list_add(list_head *enew,list_head *head){
	__list_add(enew, head, head->next);
}

/**
 * list_add_tail - add a new entry
 * @new: new entry to be added
 * @head: list head to add it before
 *
 * Insert a new entry before the specified head.
 * This is useful for implementing queues.
 */
inline void list_add_tail(list_head *enew, list_head *head){
	__list_add(enew, head->prev, head);
};

/*
 * Delete a list entry by making the prev/next entries
 * point to each other.
 *
 * This is only for internal list manipulation where we know
 * the prev/next entries already!
 */
inline void __list_del(list_head * prev, list_head * next){
	next->prev = prev;
	prev->next = next;
};

/**
 * list_del - deletes entry from list.
 * @entry: the element to delete from the list.
 * Note: list_empty on entry does not return true after this, the entry is in an undefined state.
 */
inline void list_del(list_head *entry){
	__list_del(entry->prev, entry->next);
};

/**
 * list_del_init - deletes entry from list and reinitialize it.
 * @entry: the element to delete from the list.
 */
inline void list_del_init(list_head *entry){
	__list_del(entry->prev, entry->next);
	INIT_LIST_HEAD(entry); 
};

/**
 * list_empty - tests whether a list is empty
 * @head: the list to test.
 */
inline int list_empty(const list_head *head){
	return head->next == head;
};

/**
 * list_splice - join two lists
 * @list: the new list to add.
 * @head: the place to add it in the first list.
 */
inline void list_splice(list_head *list, list_head *head){
	list_head *first = list->next;

	if (first != list) {
		list_head *last = list->prev;
		list_head *at = head->next;

		first->prev = head;
		head->next = first;

		last->next = at;
		at->prev = last;
	}
}

/**
 * list_entry - get the struct for this entry
 * @ptr:	the &struct list_head pointer.
 * @type:	the type of the struct this is embedded in.
 * @member:	the name of the list_struct within the struct.
 */
#define list_entry(ptr, type, member) \
	((type *)((char *)(ptr)-(unsigned long)(&((type *)0)->member)))

/**
 * list_for_each	-	iterate over a list
 * @pos:	the &struct list_head to use as a loop counter.
 * @head:	the head for your list.
 */
#define list_for_each(pos, head) \
	for (pos = (head)->next; pos != (head); pos = pos->next)


#endif /* __LIST_H */

